/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package opennlp.tools.util;

import java.io.IOException;
import java.util.Objects;

/**
 * Abstract base class for filtering {@link ObjectStream streams}.
 * <p>
 * Filtering streams take an existing stream and convert
 * its output to something else.
 *
 * @param <S> The sample type of the source or input stream.
 * @param <T> The sample type of this stream
 */
public abstract class FilterObjectStream<S, T> implements ObjectStream<T> {

  protected final ObjectStream<S> samples;

  /**
   * Initializes an {@link FilterObjectStream}.
   *
   * @param samples The {@link ObjectStream<S> stream} of samples to filter.
   *                Must not be {@code null}.
   */
  protected FilterObjectStream(ObjectStream<S> samples) {
    this.samples = Objects.requireNonNull(samples, "samples must not be null!");
  }

  @Override
  public void reset() throws IOException, UnsupportedOperationException {
    samples.reset();
  }

  @Override
  public void close() throws IOException {
    samples.close();
  }
}
