/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package opennlp.tools.util;

import java.io.IOException;

/**
 * A base {@link ObjectStream} implementation.
 *
 * @param <T> The generic type representing samples in the stream of samples.
 */
public abstract class AbstractObjectStream<T> implements ObjectStream<T> {

  private final ObjectStream<T> stream;

  /**
   * Initializes an {@link AbstractObjectStream}.
   *
   * @param stream The {@link ObjectStream} that provides the {@link T} samples.
   */
  protected AbstractObjectStream(ObjectStream<T> stream) {
    this.stream = stream;
  }

  @Override
  public T read() throws IOException {
    return stream.read();
  }

  @Override
  public void reset() throws IOException, UnsupportedOperationException {
    stream.reset();
  }

  @Override
  public void close() throws IOException {
    stream.close();
  }
}
